import unittest

from AptUrl import Parser


class UrlTests(unittest.TestCase):
    mapping = { 'kernel':"2.6.23-17-generic",
                'distro':"hardy" }

    testValues = ( ('apt:3ddesktop', [{'package':'3ddesktop',
                                      'schema': 'apt'}] ),
                   ('apt://3dchess', [{'package':'3dchess',
                                      'schema': 'apt'}] ),
                   ('apt+http://launchpad.net/~mvo/ppa?package=2vcard',
                    [{'package':'2vcard', 'schema': 'apt+http',
                     'repo_url': 'http://launchpad.net/~mvo/ppa',
                     'dist': '/' }] ),
                   ('apt+http://archive.canonical.com?package=acroread?dist=feisty?section=commercial',
                    [{'package':'acroread', 'schema': 'apt+http',
                     'repo_url': 'http://archive.canonical.com',
                     'dist': 'feisty', 'section': ['commercial']}] ),
                   ('apt+http://archive.canonical.com?package=acroread?keyfile=ppa-key',
                    [{'package':'acroread', 'schema': 'apt+http',
                     'repo_url': 'http://archive.canonical.com',
                     'keyfile': 'ppa-key' }]),
                   ('apt:2vcard?minver=1.0',
                    [{'package':'2vcard', 'schema': 'apt',
                     'minver': '1.0' }]),
                   ('apt:owl,newbiedoc,python-pykaraoke?dist=gutsy',
                    [{'package':'owl', 'schema':'apt'},
                    {'package':'newbiedoc', 'schema':'apt'},
                    {'package':'python-pykaraoke', 'schema':'apt', 'dist':'gutsy'}]),
                   ('apt:python-pykaraoke?dist=gutsy,owl,newbiedoc',
                    [{'package':'python-pykaraoke', 'schema':'apt', 'dist':'gutsy'},
                    {'package':'owl', 'schema':'apt'},
                    {'package':'newbiedoc', 'schema':'apt'}]),
                   ('apt:cdda2wav?section=multiverse',
                    [{'package':'cdda2wav', 'section':['multiverse']}]),
                   ('apt:java?section=multiverse?section=universe',
                    [{'package':'java', 'section':['multiverse','universe']}]),
                   ('apt:adobe-flashplugin?channel=partner',
                    [{'package':'adobe-flashplugin', 'channel': 'partner'}]),
                   ('apt:adobe-flashplugin?channel=$distro-partner',
                    [{'package':'adobe-flashplugin', 'channel': 'hardy-partner'}]),
                   ('apt:linux-restricted-modules-$kernel',
                    [{'package':'linux-restricted-modules-2.6.23-17-generic'}]),
                   ('apt:java?section=multiverse&section=universe',
                    [{'package':'java', 'section':['multiverse','universe']}]),
                   ('apt:java?section=multiverse&section=universe&minver=1.0',
                    [{'package':'java',
                      'section':['multiverse','universe'],
                      'minver': "1.0" }]),
                   ('apt:dpkg:i386', [{ 'package' : 'dpkg:i386',
                                      }]),
                 )

    def test_valid_parse(self):
        for v, e in self.testValues:
            #print("Testing: '%s'" % v)
            result_list = Parser.parse(v, self.mapping)
            for dictionary in e:
                for key in dictionary:
                    self.assertEqual(getattr(result_list[e.index(dictionary)],
                                             key), dictionary[key])

    def test_pkgname_too_long(self):
        self.assertRaises(
            Parser.InvalidUrlException, Parser.parse,
            "apt:" + 100 * "veryverylong")




from .Helpers import _, _n

class AbstractUI(object):
    # generic dialogs
    def error(self, summary, msg):
        return False
    def yesNoQuestion(self, summary, msg, title, default='no'):
        pass
    def message(self, summary, msg):
        return True
    
    # specific dialogs
    def askEnableSections(self, sections):
        " generic implementation, can be overridden "
        return self.yesNoQuestion(_("Enable additional components"),
                                  _n("Do you want to enable the following "
                                         "component: '%s'?",
                                         "Do you want to enable the following "
                                         "components: '%s'?",
                                         len(sections)) % ", ".join(sections))
    def askEnableChannel(self, channel, channel_info_html):
        " generic implementation, can be overridden "
        return self.yesNoQuestion(_("Enable additional software channel"),
                                  _("Do you want to enable the following "
                                    "software channel: '%s'?") % channel)
    def askInstallPackage(self):
        pass

    # install/update progress 
    def doUpdate(self):
        pass
    def doInstall(self, pkglist):
        pass

    # UI specific actions for enabling stuff

    # FIXME: the next two functions shoud go into generic code
    #        that checks for the availablility of tools
    #        like gksu or kdesudo and uses them 
    #        appropriately
    def doEnableSection(self, sections):
        pass
    def doEnableChannel(self, channelpath, channelkey):
        pass



# -*- coding: utf-8 -*-

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtWebKit import QWebView
from PyQt4 import uic
from PyKDE4.kdeui import *
from PyKDE4.kdecore import *

import os
import apt_pkg
import subprocess

from AptUrl.UI import AbstractUI
from AptUrl.Helpers import _

APTURL_UI_FILE = os.environ.get(
    # Set this envar to use a test .ui file.
    'APTURL_UI_FILE',
    # System file to use if the envar is not set.
    '/usr/share/apturl/apturl-qt.ui'
    )


class AptUrlDialog(KDialog):
    def __init__(self,parent=None):
        KDialog.__init__(self,parent)
        self.setWindowIcon(KIcon("application-x-deb"))

    def slotButtonClicked(self,button):
        if button in (KDialog.Apply, KDialog.Ok, KDialog.Yes):
            KDialog.accept(self)
        else:
            KDialog.reject(self)


class KdeUI(AbstractUI):
    def __init__(self):
        self.dialog = AptUrlDialog()

        self.d = QWidget(self.dialog)
        self.dialog.setMainWidget(self.d)

        uic.loadUi(APTURL_UI_FILE, self.d)
        self.d.image_label.setPixmap(KIcon("application-x-deb").pixmap(64,64))

    # generic dialogs
    def _get_dialog(self, dialog_type, summary, msg="", buttons=KDialog.Close):
        self.dialog.setButtons(KDialog.ButtonCode(buttons))
        self.d.header_label.setText("<h2>" + summary + "</h2>")
        self.d.body_label.setText(msg)
        # TODO: title = empty
        # TODO: keep above maintain across events
        # TODO: d.set_markup("<big><b>%s</b></big>\n\n%s" % (summary, msg))

    def error(self, summary, msg=""):
        if msg != "":
            KMessageBox.detailedError(AptUrlDialog(), summary, msg)
        else:
            KMessageBox.error(AptUrlDialog(), summary)
        return False

    def message(self, summary, msg="", title=""):
        if msg != "":
            summary += "\n\n" + msg
        KMessageBox.information(AptUrlDialog(), summary, title)
        return True

    def yesNoQuestion(self, summary, msg, title="", default='no'):
        self._get_dialog('', summary, msg,
                             buttons=KDialog.Yes | KDialog.No)
        self.d.setWindowTitle(title)
        res = self.dialog.exec_()
        if res != KDialog.Accepted:
            return False
        return True

    # specific dialogs
    def askEnableChannel(self, channel, channel_info_html):
        summary = "<h2>" + _("Enable additional software channel") + "</h2>"
        msg = _("Do you want to enable the following "
                "software channel: '%s'?") % channel
        self._get_dialog('', summary, msg,
                             buttons=KDialog.Yes | KDialog.No)
        webview = QWebView()
        webview.setHtml(channel_info_html)
        webview.setFixedSize(400,200)
        self.d.vlayout_right.addWidget(webview)
        res = self.dialog.exec_()
        if res != KDialog.Accepted:
            return False
        return True

    def doEnableSection(self, sections):
        cmd = ["kdesudo",
               "--",
               "software-properties-kde",
               "--enable-component", "%s" % ' '.join(sections)]
        try:
            output = subprocess.check_output(
                cmd,
                stdout=subprocess.PIPE,
                universal_newlines=True)
        except (OSError, subprocess.CalledProcessError) as e:
            print("Execution failed: %s" % e, file=sys.stderr)
            return True
        # FIXME: Very ugly, but kdesudo doesn't return the correct exit states
        print(output)
        if not output.startswith("Enabled the "):
            return False
        return True


    def doEnableChannel(self, channelpath, channelkey):
        cmd = ["kdesudo",
               "--",
               "install", "--mode=644","--owner=0",channelpath,
               apt_pkg.Config.FindDir("Dir::Etc::sourceparts")]
        res = subprocess.call(cmd, universal_newlines=True)
        if res != 0:
            return False
        # install the key as well
        if os.path.exists(channelkey):
            cmd = ["kdesudo",
                   "--",
                   "apt-key", "add",channelkey]
            res = subprocess.call(cmd, universal_newlines=True)
            if res != 0:
                return False
        return True

    def askInstallPackage(self, package, summary, description, homepage):
        header = "<h2>" + _("Install additional software?") + "</h2>"
        body = _("Do you want to install package '%s'?") % package
        self.d.header_label.setText(header)
        self.d.body_label.setText(body)

        self.dialog.setButtons(KDialog.ButtonCode(KDialog.Yes | KDialog.No))

        res = self.dialog.exec_()
        if res != KDialog.Accepted:
            return False
        return True

    # progress etc
    def doUpdate(self):
        subprocess.check_call([
            'qapt-batch',
            '--attach', str(self.dialog.winId()),
            '--update'
            ],
            universal_newlines=True)

    def doInstall(self, apturl):
        subprocess.check_call([
            'qapt-batch',
            '--attach', str(self.dialog.winId()),
            '--install',
            apturl.package
            ],
            universal_newlines=True)


if __name__ == "__main__":
    ui = KdeUI()
    ui.error("foo","bar")

# kate: space-indent on; indent-width 4; mixedindent off; indent-mode python;



#!/usr/bin/env python3

from distutils.core import setup
from DistUtilsExtra.command import *
import glob
import os
import re

# look/set what version we have
changelog = "debian/changelog"
if os.path.exists(changelog):
    with open(changelog, encoding='utf-8') as fp:
        head = fp.readline()
    match = re.compile(".*\((.*)\).*").match(head)
    if match:
        version = match.group(1)
        with open("AptUrl/Version.py", "w") as fp:
            print('VERSION="{}"'.format(version))

GETTEXT_NAME="apturl"
I18NFILES = []
for filepath in glob.glob("po/mo/*/LC_MESSAGES/*.mo"):
    lang = filepath[len("po/mo/"):]
    targetpath = os.path.dirname(os.path.join("share/locale",lang))
    I18NFILES.append((targetpath, [filepath]))

setup(name='apturl',
      version=version,
      packages=['AptUrl',
                'AptUrl.gtk',
                'AptUrl.gtk.backend',
                'AptUrl.kde'],
      scripts=['apturl','apturl-gtk','apturl-kde'],
      data_files=[('share/apturl/',
                   ["data/apturl-gtk.ui","data/apturl-qt.ui"]),
                  ('../etc/firefox/pref/',
                   ["data/apturl.js"]),
                  ('share/kde4/services/',
                   ["data/apt.protocol","data/apt+http.protocol"]),
                  ('share/applications/',
                   ["data/apturl.desktop"]),
                 ]+I18NFILES,
      cmdclass = { "build" : build_extra.build_extra,
                   "build_i18n" :  build_i18n.build_i18n,
                   "build_help" :  build_help.build_help,
                   "build_icons" :  build_icons.build_icons }
      )
